@@ -50,7 +50,8 @@ bootdata.trans.msm <- function(x) {
     data.boot <- as.data.frame(data.boot)
     ## label every transition in new data as from a different subject
     data.boot[,un["subject"]] <- rep(1:ntrans, each=2)
-    for (i in which(sapply(data.boot, is.factor)))
+
+    for (i in names(dat)[sapply(dat, is.factor)])
         data.boot[,i] <- factor(data.boot[,i], labels=sort(unique(dat[,i])))
     data.boot
 }
@@ -227,6 +228,20 @@ expected.ci.msm <- function(x,
     res
 }
 
+updatepars.msm <- function(x, pars){
+    x.rep <- x
+    x.rep$paramdata$params <- pars
+    x.rep$estimates <- pars
+    output <- msm.form.output(x.rep, "intens")
+    x.rep$Qmatrices <- output$Qmatrices
+    if (x$emodel$misc) {
+        output <- msm.form.output(x.rep, "misc")
+        x.rep$Ematrices <- output$Ematrices
+        names(x.rep$Ematrices)[1] <- "logitbaseline"
+    }
+    x.rep    
+}
+
 ### Compute a CI for a statistic using a sample from the assumed MVN
 ### distribution of MLEs of log Q, logit E and covariate effects on these
 ### Not user visible: only support statistics based on Q matrix and E matrix
@@ -239,21 +254,13 @@ normboot.msm <- function(x, stat, B=1000) {
     params <- matrix(nrow=B, ncol=x$paramdata$npars)  # replicate constrained parameters.
     params[,x$paramdata$optpars] <- sim
     params[,x$paramdata$fixedpars] <- rep(x$paramdata$params[x$paramdata$fixedpars], each=B)
-    params[,x$paramdata$hmmpars] <- rep(msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel$plabs, x$hmodel$parstate), each=B)
+    params[,x$paramdata$hmmpars] <- rep(msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel), each=B)
     params <- params[, !duplicated(abs(x$paramdata$constr)), drop=FALSE][, abs(x$paramdata$constr), drop=FALSE] *
         rep(sign(x$paramdata$constr), each=B)
 
     sim.stat <- vector(B, mode="list")
     for (i in 1:B) {
-        x.rep <- x
-        x.rep$paramdata$params <- params[i,]
-        output <- msm.form.output(x.rep, "intens")
-        x.rep$Qmatrices <- output$Qmatrices
-        if (x$emodel$misc) {
-            output <- msm.form.output(x.rep, "misc")
-            x.rep$Ematrices <- output$Ematrices
-            names(x.rep$Ematrices)[1] <- "logitbaseline"
-        }
+        x.rep <- updatepars.msm(x, params[i,])
         sim.stat[[i]] <- stat(x.rep)
     }
     sim.stat
